###############################################################
### Example, Section 6.1 - part II (standardisation)
### Cyclists hospitalized with a head injury
### Vencalek, Hron, Filzmoser

### libraries: VGAM 
### data:      accidents-head.csv, age-auxiliary.csv
### R-files:   RealData-Functions.R
###############################################################

library(VGAM)

### Data - accidents
  x = read.csv2("accidents-head.csv",header=FALSE)
  names(x) = c("year","group1","group2","group3")

### Data - population
  xx = read.csv2("age-auxiliary.csv",header=FALSE)
  names(xx) = c("year","group1","group2","group3")
  xx = xx[xx$year %in% x$year, ]
  
  for (i in 2:4)
    { x[,i] = as.numeric(x[,i])
     xx[,i] = as.numeric(xx[,i])}

### Functions  
  source("RealData-Functions.R")
  
  Years = dim(x)[1]
  Start = x[1,1]
  
#######################################################
  ### Standardization
#######################################################

  xs = x
  for (i in 1:11)
    xs[i,2:4] = round( x[i,2:4]*xx[11,2:4]/xx[i,2:4]*sum(xx[i,2:4])/sum(xx[11,2:4]) )
  x = xs
  rm(xs)
  
  coordinates = coord.adj(selected.data(x,Start,Years)[,-1])
  
#######################################################
   ### Models
#######################################################
  
  ### parameters + CIs 
  # (a1,a2,b1,b2)
  # column 1: point estimation
  # columns 2-3: 95% CI
  print(coda.param <- my.coda.adj(selected.data(x,Start,Years)[,-1],1:Years-1,typ="CI"))
  print(glm.param  <- my.glm.adj(selected.data(x,Start,Years)[,-1],1:Years-1,typ="CI"))
  
  ### models:
  model.coda1 = my.coda.adj(selected.data(x,Start,Years)[,-1],1:Years-1,typ="model",ktery=1)
  model.coda2 = my.coda.adj(selected.data(x,Start,Years)[,-1],1:Years-1,typ="model",ktery=2)
  model.glm   =  my.glm.adj(selected.data(x,Start,Years)[,-1],1:Years-1,typ="model")
  ### model with proportional odds
  tim = 1:Years-1
  model.glmB  =  vglm( cbind(group1,group2,group3) ~ tim, 
                               family=acat(reverse=TRUE, parallel=TRUE), 
                               selected.data(x,Start,Years)[,-1])
  ### Validity of prop. odds assumption:
  lrtest(model.glm, model.glmB)
  

  
### TABLE 3:  
  ### Comparison of the point estimates:
  comp1 = data.frame(coda.param[,1],glm.param[,1],coda.param[,1]-glm.param[,1])
  names(comp1) = c("coda","glm","difference")
  print("Comparison of the point estimates:")
  print(comp1)
  
  
  ### Comparison of the standard deviations:
  sd.coda = (coda.param[,3] - coda.param[,2])/(2*qt(0.975,Years-2))
  sd.glm  = ( glm.param[,3] -  glm.param[,2])/(2*qnorm(0.975))
  comp2 = data.frame(sd.coda,sd.glm,sd.coda/sd.glm)
  names(comp2) = c("coda","glm","ratio")
  print("Comparison of the standard deviations:")
  print(comp2)
  
  
#######################################################
  ### Graphics
#######################################################
  
  
  ### barplot
  #pdf("graf-accidents-standard.pdf",height=4.5,width = 7.5)
    par(xpd=TRUE, mar=c(6,3,1,1),mfrow=c(1,1))
    graf(selected.data(x,Start,Years),barvy=heat.colors(3))
    legend(x=3,y=-0.2,horiz=TRUE, legend=c("0-14 years","15-26 years","older than 26 years"),fill=heat.colors(3));
  #dev.off()
  
  ### coordinates and fit  
  #pdf("graf-nehody-fit.pdf",height=5,width = 11)
    popis = c('1999',"'00","'01","'02","'03","'04","'05","'06","'07","'08",'2009')
    par(mfrow=c(1,2),mar=c(4,5,1,1))
  
    plot(coordinates$z1,pch=20,axes=FALSE,frame=TRUE,
         xlab = "year", ylab = expression(paste("empirical log(", pi[1]/pi[2],")")))
    axis(1,1:11,popis)
    axis(2)
    points(model.coda1$fit,type="l",col=2,lwd=2)
    points(coord(model.glm@fitted.values)$z1,type="l",col=4,lwd=2)
    
    plot(coordinates$z2,pch=20,axes=FALSE,frame=TRUE,
         xlab = "year", ylab = expression(paste("empirical log(", pi[2]/pi[3],")")))
    axis(1,1:11,popis)
    axis(2)
    points(model.coda2$fit,type="l",col=2,lwd=2)
    points(coord.adj(model.glm@fitted.values)$z2,type="l",col=4,lwd=2)
  #dev.off()
  