###############################################################
### Functions used in examples, Section 6
### Vencalek, Hron, Filzmoser

# coord ... computes orthogonal coordinates
# coord.adj ... computes adjacent coordinates
# selected.data ... selection of data according to the 1st column (containing time information)
# graf ... plots stacked barplot
# my.coda ... fits CoDa model for orthogonal coordinates
# my.glm  ... fits GLM  model for orthogonal coordinates
# my.coda.adj ... fits CoDa model for adjacent coordinates
# my.glm.adj  ... fits GLM  model for adjacent coordinates


################################################################
### computation of the orthogonal coordinates
################################################################


coord = function(x)
{
  pi = x/apply(x,1,sum)
  z1 = log(pi[,1]/pi[,2])
  z2 = log(sqrt(pi[,1]*pi[,2])/pi[,3])   
  return(list(z1=z1,z2=z2))
}

################################################################
### computation of the adjacent coordinates
################################################################


coord.adj = function(x)
{
  pi = x/apply(x,1,sum)
  z1 = log(pi[,1]/pi[,2])
  z2 = log(pi[,2]/pi[,3])   
  return(list(z1=z1,z2=z2))
}

################################################################
### selection of data according to the 1st column (containing time information)
################################################################

selected.data = function(data,od,delka)
{
  ### od = from (year)
  ### delka = length of the considered period
  pom.data = data[data[,1] %in% od:(od+delka-1),]
  return(pom.data)
}


################################################################
### barplot
################################################################


graf = function(data,barvy,popis="")
{
  ### barvy = colours
  ### popis = label of the y-axis
  pomdata = data.frame(data[,-1])
  rownames(pomdata) = data[,1]
  tab1 = prop.table(as.table(as.matrix(pomdata)),1)
  barplot(t(tab1),col=barvy,ylab=popis)
}


################################################################
### CoDa model -  ORTHOGONAL COORDINATES
################################################################

my.coda = function(x,tim,typ,ktery=1)
{
  ### x     ... data matrix: 1 row for 1 year, 3 columns (i.e. 3 groups) expected
  ### tim   ... time 
  ### typ   ... what should be returned: "CI" or "coef" or "var" or "model"
  ### ktery ... 1 = the model for the 1st coordinate, 2 = the model for the 2nd coordinate

  ### coordinates
  pi = x/apply(x,1,sum)
  z1 = log(pi[,1]/pi[,2])
  z2 = log(sqrt(pi[,1]*pi[,2])/pi[,3])  
  
  ### linear model for the 1st coordinate
  m1 = lm(z1~tim)
  k1 = as.numeric(m1$coef)
  V1 = summary(m1)$sigma^2*summary(m1)$cov
  s1 = sqrt(diag(V1))
  
  ### linear model for the 2nd coordinate
  m2 = lm(z2~tim)
  k2 = as.numeric(m2$coef)
  V2 = summary(m2)$sigma^2*summary(m2)$cov
  s2 = sqrt(diag(V2))
  
  ### confidence intervals
  CI.coda = matrix(ncol=3,nrow=4)
  CI.coda[,1] = c(k1,k2)
  CI.coda[,2] = c(k1,k2) - qt(0.975,length(z1)-2)*c(s1,s2)
  CI.coda[,3] = c(k1,k2) + qt(0.975,length(z1)-2)*c(s1,s2)
  CI.coda = CI.coda[c(1,3,2,4),]
  
  if (typ=="CI") return(CI.coda)
  if (typ=="coef") return(list(k1,k2))
  if ((typ=="model")&(ktery==1)) return(m1)
  if ((typ=="model")&(ktery==2)) return(m2)
  if (typ=="var") return(list(s1,s2,V1,V2))
}  

################################################################
### GLM model -  ORTHOGONAL COORDINATES
################################################################

my.glm = function(x,tim,typ)
{
  ### x     ... data matrix: 1 row for 1 year, 3 columns (i.e. 3 groups) expected
  ### tim   ... time 
  ### typ   ... what should be returned: "CI" or "coef" or "var" or "model"
  
  ### model:
  fit1 = vglm( cbind(group1,group2,group3) ~ tim, 
               family=acat(reverse=TRUE, parallel=FALSE), x)
  
  ### coeficients:
  k = summary(fit1)@coefficients
  V = vcov(fit1)
  PC = matrix(c(1,0.5,0,0,0,1,0,0,0,0,1,0.5,0,0,0,1),ncol=4) 
  V = PC %*% V %*% t(PC)                        
  s = sqrt(diag(V))
  k = c(k[1],k[1]/2+k[2],k[3],k[3]/2+k[4]) 

  ### confidence intervals:
  CI.glm = matrix(ncol=3,nrow=4)
  CI.glm[,1] = k
  CI.glm[,2] = k - qnorm(0.975)*s
  CI.glm[,3] = k + qnorm(0.975)*s
  
  if (typ=="CI") return(CI.glm)
  if (typ=="coef") return(k)
  if (typ=="model") return(fit1)
  if (typ=="var") return(list(s,V))
}  

################################################################
### CoDa model -  ADJACENT COORDINATES
################################################################

my.coda.adj = function(x,tim,typ,ktery=1)
{
  ### coordinates
  pi = x/apply(x,1,sum)
  z1 = log(pi[,1]/pi[,2])
  z2 = log(pi[,2]/pi[,3])
  
  ### linear model for the 1st coordinate
  m1 = lm(z1~tim)
  k1 = as.numeric(m1$coef)
  V1 = summary(m1)$sigma^2*summary(m1)$cov
  s1 = sqrt(diag(V1))
  
  ### linear model for the 2nd coordinate
  m2 = lm(z2~tim)
  k2 = as.numeric(m2$coef)
  V2 = summary(m2)$sigma^2*summary(m2)$cov
  s2 = sqrt(diag(V2))
  
  ### confidence intervals
  CI.coda = matrix(ncol=3,nrow=4)
  CI.coda[,1] = c(k1,k2)
  CI.coda[,2] = c(k1,k2) - qt(0.975,length(z1)-2)*c(s1,s2)
  CI.coda[,3] = c(k1,k2) + qt(0.975,length(z1)-2)*c(s1,s2)
  CI.coda = CI.coda[c(1,3,2,4),]
  
  if (typ=="CI") return(CI.coda)
  if (typ=="coef") return(list(k1,k2))
  if ((typ=="model")&(ktery==1)) return(m1)
  if ((typ=="model")&(ktery==2)) return(m2)
  if (typ=="var") return(list(s1,s2,V1,V2))
}  

################################################################
### GLM model -  ADJACENT COORDINATES
################################################################

my.glm.adj = function(x,tim,typ)
{
  
  ### model:
  fit1 = vglm( cbind(group1,group2,group3) ~ tim, 
               family=acat(reverse=TRUE, parallel=FALSE), x)
  
  ### coeficients:
  k = summary(fit1)@coefficients
  V = vcov(fit1)
  s = sqrt(diag(V))
  
  ### confidence intervals:
  CI.glm = matrix(ncol=3,nrow=4)
  CI.glm[,1] = k
  CI.glm[,2] = k - qnorm(0.975)*s
  CI.glm[,3] = k + qnorm(0.975)*s
  
  if (typ=="CI") return(CI.glm)
  if (typ=="coef") return(k)
  if (typ=="model") return(fit1)
  if (typ=="var") return(list(s,V))
}  

################################################################


