###############################################################
### Example, Section 6.2
### Age structure of the Czech Republic
### Vencalek, Hron, Filzmoser

### libraries: VGAM, MASS
### data:      age-basic.csv
### R-files:   RealData-Functions.R, RealData-Functions2.R
###############################################################

library(VGAM)
library(MASS)

x = read.csv2("age-basic.csv",header=FALSE)
names(x) = c("year","group1","group2","group3")

source("RealData-Functions.R")
source("RealData-Functions2.R")

################################################################
### models and point predictions
################################################################

Tim = x[,1]
Coordinates = coord.adj(x[,-1])
LengthTraining = 3

pc1 = numeric() # prediction Coda, coord.1
pc2 = numeric() # prediction Coda, coord.2
pg1 = numeric() # prediction Glm,  coord.1
pg2 = numeric() # prediction Glm , coord.2

TimPred = Tim[-c(1:LengthTraining)]

for (i in 1:length(TimPred))
{
  Start = Tim[1]+i-1
  tim = Start:(Start+LengthTraining-1)

### models:
  model.coda1 = my.coda.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model",ktery=1)
  model.coda2 = my.coda.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model",ktery=2)
  model.glm   =  my.glm.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model")

  pc1[i] = predict(model.coda1,new=list(tim=LengthTraining))
  pc2[i] = predict(model.coda2,new=list(tim=LengthTraining))
  pg1[i] = predictvglm(model.glm,newdata=list(tim=LengthTraining))[1]
  pg2[i] = predictvglm(model.glm,newdata=list(tim=LengthTraining))[2]
}

################################################################
### graphics
################################################################

### FIGURE 5:
  #pdf("graf-population.pdf",height=4.5,width = 7.5)
    par(mfrow=c(1,1))
    Barvy = c("gray25","gray50","gray85")
    par(xpd=TRUE, mar=c(6,4.4,1,1))
    graf(x,barvy=Barvy,"Proportion")
    legend(x=5,y=-0.2,horiz=TRUE, legend=c("0-14 years","15-64 years","65+ years"),fill=Barvy)
  #dev.off()


### FIGURE 6    
  #pdf("graf-population-coordinates.pdf",height=4,width = 12)
  par(mfrow=c(1,2),mar=c(4,5,1,1))
  
    plot(Coordinates$z1~Tim,pch=20,cex.lab=1.3,
         xlab = "year", ylab = expression(paste("empirical ln( ", pi[1]/pi[2],")")))
    plot(Coordinates$z2~Tim,pch=20,cex.lab=1.3,
         xlab = "year", ylab = expression(paste("empirical ln( ", pi[2]/pi[3],")")))
  #dev.off()


### unpublished figure:
  #pdf("graf-population-predictions.pdf",height=5,width = 11)
  par(mfrow=c(1,2),mar=c(4,5,1,1))
  
  plot(Coordinates$z1~Tim,pch=20,
       xlab = "year", ylab = expression(paste("empirical ln(", pi[1]/pi[2],")")))
    points(pc1~TimPred,col=3)
    points(pg1~TimPred,col=2)
  
  plot(Coordinates$z2~Tim,pch=20,
       xlab = "year", ylab = expression(paste("empirical ln(", pi[2]/pi[3],")")))
    points(pc2~TimPred,col=3)
    points(pg2~TimPred,col=2)
  #dev.off()


################################################################
  ### prediction errors
################################################################
    
### absolute prediction errors:
  abs.error.c1 = abs(Coordinates$z1[(LengthTraining+1):25] - pc1)
  abs.error.c2 = abs(Coordinates$z2[(LengthTraining+1):25] - pc2)
  abs.error.g1 = abs(Coordinates$z1[(LengthTraining+1):25] - pg1)
  abs.error.g2 = abs(Coordinates$z2[(LengthTraining+1):25] - pg2)


# 1st coordinate:
  sum(abs.error.c1)/sum(abs.error.g1)  
# 2nd coordinate:
  sum(abs.error.c2)/sum(abs.error.g2)  


################################################################
  ### prediction intervals
################################################################

  ### 1 - computation:
  
  
se = matrix(ncol=4,nrow = 25-LengthTraining)
PI = matrix(ncol=8,nrow = 25-LengthTraining)

N = 10000

for (i in 1:length(TimPred))
{
  Start = Tim[1]+i-1

  coda.param <- my.coda.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="CI")
  glm.param  <-  my.glm.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="CI")
  
  ### Comparison of the standard deviations:
  sd.coda = (coda.param[,3] - coda.param[,2])/(2*qt(0.975,LengthTraining-2))
  sd.glm  = ( glm.param[,3] -  glm.param[,2])/(2*qnorm(0.975))
  comp2 = data.frame(sd.coda,sd.glm,sd.coda/sd.glm)
  names(comp2) = c("coda","glm","ratio")
  print(paste("Comparison of the standard deviations, year ",Start))
  print(comp2)

  model.coda1 = my.coda.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model",ktery=1)
  model.coda2 = my.coda.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model",ktery=2)
  model.glm   =  my.glm.adj(selected.data(x,Start,LengthTraining)[,-1],1:LengthTraining-1,typ="model")
  
  se[i,1]   = predict(model.coda1  , new=list(tim=LengthTraining), se.fit=TRUE)$se.fit
  se[i,2]   = predict(model.coda2  , new=list(tim=LengthTraining), se.fit=TRUE)$se.fit
  se[i,3:4] = predictvglm(model.glm, new=list(tim=LengthTraining), se.fit=TRUE)$se.fit

  oN = apply(x,1,sum)[Start+LengthTraining-Tim[1]]

  pig = pred.interval.glm(model.glm)

  PI[i,1:2] = predict(model.coda1  , new=list(tim=LengthTraining), interval="prediction")[2:3]
  PI[i,3:4] = predict(model.coda2  , new=list(tim=LengthTraining), interval="prediction")[2:3]
  PI[i,5:6] = pig$ip.z1 
  PI[i,7:8] = pig$ip.z2 
}

length.PI.coda2 = PI[,4]-PI[,3]
length.PI.coda1 = PI[,2]-PI[,1]
length.PI.glm1  = PI[,6]-PI[,5]
length.PI.glm2  = PI[,8]-PI[,7]


  ### 2 - RESULTS #######

  ### 1st coordinate
    cbind(TimPred,length.PI.coda1,length.PI.glm1,length.PI.coda1/length.PI.glm1)
    range(length.PI.coda1)
    range(length.PI.glm1)
    podil = length.PI.coda1/length.PI.glm1
    c(1993:2014)[which(podil==max(podil))]
    max(podil)
    sum(podil>1)
    sum(podil<1)

  ### 2nd coordinate    
    cbind(TimPred,length.PI.coda2,length.PI.glm2,length.PI.coda2/length.PI.glm2)
    range(length.PI.coda2)
    range(length.PI.glm2)
    podil = length.PI.coda2/length.PI.glm2
    c(1993:2014)[which(podil==max(podil))]
    max(podil)
    sum(podil>1)
    sum(podil<1)
  

  ### 3 - GRAPHICS #######
  
  ### FIGURE 7    
  #pdf("graf-population-errors.pdf",height=7.5,width = 7.5)
    par(mfrow=c(2,1),mar=c(4,5,1,1))
    
    plot(abs.error.c1 ~ I(length.PI.coda1/2),pch=20,
         ylab="absolute prediction error", xlab = "half-length of the prediction interval",
         xlim = c(0,0.16), ylim = c(0,0.023))
      points(abs.error.g1 ~ I(length.PI.glm1/2),pch=4,cex=0.6)
      lines(c(0,1),c(0,1))
    
    plot(abs.error.c2 ~ I(length.PI.coda2/2),pch=20,
         ylab="absolute prediction error", xlab = "half-length of the prediction interval",
         xlim = c(0,0.16), ylim = c(0,0.023))
    points(abs.error.g2 ~ I(length.PI.glm2/2),pch=4,cex=0.6)
      lines(c(0,1),c(0,1))
  #dev.off()




